"""sdl2 extensions.

This package provides easy access to the most common used video and graphics
functionality of the SDL2 bindings. It enables you to easily create windows,
display on them and to manipulate the shown graphics.
"""
from .algorithms import *
from .array import *
from .color import *
from .resources import *
from .events import *
from .ebs import *

from .common import *
from .draw import *
from .font import *
from .gui import *
from .image import *
from .pixelaccess import *
from .sprite import *
from .surface import *
from .window import *
